
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% now we do it with epstein--zin preferences with potentially 
%% time-varying risk aversion

clear all

dep=.02;
gamma=.33;
rho=1/2;
beta=-log(.99);
phi=.9;
DIS=.4;

rbar=beta;
Kbar=((exp(beta)-1+dep)/gamma)^(1/(gamma-1));
% Kbar=Kbar*exp(.078);

kbar=log(Kbar);
Ibar = dep*Kbar;
cbar = log((Kbar^gamma) - Ibar);

wbar=-log(exp(rbar)-1);
phi_p=.92;
p_bar=.02/4;
alpha=6;

sd_p=.0085/4;
inv_p=sd_p*sqrt(1-phi_p^2);


%% these will be coordinates in the matrices GAM0 and GAM1 for gensys
c=1;
k=2;
p=3;
rw=4;
wc=5;
erw=6; %% erw, ec, and ea are expectational variables
ec=7;
CONS=8;
rf=9;

GAM0=zeros(9,9); GAM1=GAM0;

%% capital euler equation
GAM0(1,k) = gamma*(gamma-1)*exp((gamma-1)*kbar)/...
    (gamma*exp((gamma-1)*kbar)+1-dep);
GAM0(1,ec) = -rho*(1-alpha)/(1-rho);
GAM0(1,c) = rho*(1-alpha)/(1-rho);
GAM0(1,erw) = (rho-alpha)/(1-rho);
% GAM0(1,CONS)=-beta + log(gamma*exp((gamma-1)*kbar)+1-dep);

%% budget constraint
GAM0(2,c) = -exp(cbar);
GAM0(2,k) = -exp(kbar);
GAM1(2,k) = -(1-dep)*exp(kbar)-gamma*exp(gamma*kbar);

%% euler equation for wealth
GAM0(3,erw) = 1;
GAM0(3,ec) = -rho;
GAM0(3,c) = rho;

%% wealth return
GAM0(4,rw) = 1;
GAM0(4,c) = -1;
GAM1(4,c) = -1;
GAM0(4,wc) = -exp(-rbar);
GAM1(4,wc) = -1;

%% disaster probability
GAM0(5,p) = 1;
GAM1(5,p) = phi_p;
GAM0(5,CONS)=-(1-phi_p)*p_bar;

%% expectational equations 
GAM0(6,c)=1; GAM1(6,ec)=1;
GAM0(7,rw)=1; GAM1(7,erw)=1;

%% constant
GAM0(8,CONS)=1;
GAM1(8,CONS)=1;

GAM0(9,rf)=-1;
GAM0(9,c)=rho*(1-alpha)/(1-rho);
GAM0(9,ec)=-rho*(1-alpha)/(1-rho);
GAM0(9,erw)=(rho-alpha)/(1-rho);

%% innovations; scaled so that the errors have unit variance
PSI=zeros(9,2);

%% innovation to prob
PSI(5,1)=inv_p; 

%% wealth return
PSI(4,2)=-DIS;

%% expectational errors
PPI=zeros(9,2);
PPI(6:7,:)=eye(2);

%% solve the first time (first-order perturbation)
C0=zeros(9,1);
[G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(GAM0,GAM1,C0,PSI,PPI) ;

%% get an impulse response function
X=zeros(9,1); X(p)=1;
IRF=zeros(9,100);
for i=1:100
    IRF(:,i)=(G1^(i-1))*X;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% now we find what the coefficient on z should be

Gk=zeros(1,9); Gk(k)=1;
Gc=zeros(1,9); Gc(c)=1;
Gw=zeros(1,9); Gw(rw)=1;


for i=1:10

    Hed = impact(:,2);
    Hed(c)=-DIS;
   
    fed = -DIS*(1-dep)/(gamma*exp((gamma-1)*kbar)+1-dep);
    m_xp = -rho*(1-alpha)*Gc/(1-rho) + (rho-alpha)*Gw/(1-rho);
    f_xp=0; med=0;
    p_k=exp((f_xp+m_xp)*Hed + fed+med) - ...
        exp(((f_xp+m_xp)*Hed + fed+med)*(-p_bar/(1-p_bar)))*(...
         1 + inv(1-p_bar)*((f_xp+m_xp)*Hed + fed+med));
    p_k=p_k/(p_bar*exp((f_xp+m_xp)*Hed + fed+med) + ...
                (1-p_bar)*exp(((f_xp+m_xp)*Hed + fed+med)*(-p_bar/(1-p_bar)))...
              );
    z = ((f_xp+m_xp)*Hed + fed+med);
    c_k = log( p_bar*exp(z) + (1-p_bar)*exp(z*(-p_bar/(1-p_bar))) ) -p_k*p_bar;
          
%     p2=p_bar;
%     p3=p_bar+inv_p;
%     z=(f_xp+m_xp)*Hed + fed+med;
%   log(linspace(0,.9,100)*exp(z) + (1-linspace(0,.9,100)).*exp(z*(-linspace(0,.9,100))./(1-linspace(0,.9,100))))
%   log(p3*exp(z) + (1-p3)*exp(z*(-p3)/(1-p3)))
%           
    
    fed = 0;
    m_xp = -rho*(1-alpha)*Gc/(1-rho) + (rho-alpha)*Gw/(1-rho);
    f_xp=Gw;
    p_w=exp((f_xp+m_xp)*Hed + fed+med) - ...
        exp(((f_xp+m_xp)*Hed + fed+med)*(-p_bar/(1-p_bar)))*(...
         1 + inv(1-p_bar)*((f_xp+m_xp)*Hed + fed+med));
    p_w=p_w/(p_bar*exp((f_xp+m_xp)*Hed + fed+med) + ...
                (1-p_bar)*exp(((f_xp+m_xp)*Hed + fed+med)*(-p_bar/(1-p_bar)))...
              );
    z = ((f_xp+m_xp)*Hed + fed+med);
    c_w = log( p_bar*exp(z) + (1-p_bar)*exp(z*(-p_bar/(1-p_bar))) ) -p_k*p_bar;
    
    fed = 0;
    m_xp = -rho*(1-alpha)*Gc/(1-rho) + (rho-alpha)*Gw/(1-rho);
    f_xp=0;
    p_f=exp((f_xp+m_xp)*Hed + fed+med) - ...
        exp(((f_xp+m_xp)*Hed + fed+med)*(-p_bar/(1-p_bar)))*(...
         1 + inv(1-p_bar)*((f_xp+m_xp)*Hed + fed+med));
    p_f=p_f/(p_bar*exp((f_xp+m_xp)*Hed + fed+med) + ...
                (1-p_bar)*exp(((f_xp+m_xp)*Hed + fed+med)*(-p_bar/(1-p_bar)))...
              );
    z = ((f_xp+m_xp)*Hed + fed+med);
    c_f = log( p_bar*exp(z) + (1-p_bar)*exp(z*(-p_bar/(1-p_bar))) ) -p_k*p_bar;
    
%% capital euler equation
    GAM0(1,k) = gamma*(gamma-1)*exp((gamma-1)*kbar)/...
        (gamma*exp((gamma-1)*kbar)+1-dep);
    GAM0(1,ec) = -rho*(1-alpha)/(1-rho);
    GAM0(1,c) = rho*(1-alpha)/(1-rho);
    GAM0(1,erw) = (rho-alpha)/(1-rho);
    GAM0(1,p) = p_k;
%     GAM0(1,CONS) = -beta + log(gamma*exp((gamma-1)*kbar)+1-dep)+c_k;
    
    %% euler equation for wealth
    GAM0(3,erw) = (1-alpha)/(1-rho);
    GAM0(3,ec) = -rho*(1-alpha)/(1-rho);
    GAM0(3,c) = rho*(1-alpha)/(1-rho);
    GAM0(3,p)=p_w;
%     GAM0(3,CONS) = -beta+c_w;
    
    GAM0(9,rf)=1;
    GAM0(9,c)=rho*(1-alpha)/(1-rho);
    GAM0(9,ec)=-rho*(1-alpha)/(1-rho);
    GAM0(9,erw)=(rho-alpha)/(1-rho);
    GAM0(9,p)=p_f;

    disp([p_k p_w])

    [G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(GAM0,GAM1,C0,PSI,PPI) ;

end

%% get an impulse response function

IRF=zeros(9,100);
for i=1:100
    IRF(:,i)=(G1^(i-1))*impact(:,1);
end

%% steady state
SS = (G1^1000)*[ones(8,1); 1];

eps=sqrt(2)*[-5.38748089
-4.60368245
-3.94476404
-3.347854567
-2.788806058
-2.254974002
-1.738537712
-1.234076215
-0.737473729
-0.245340708
0.245340708
0.737473729
1.234076215
1.738537712
2.254974002
2.788806058
3.347854567
3.94476404
4.60368245
5.38748089
];

ww=(pi^(-1/2))*[2.2293936455400E-13
4.3993409922600E-10
1.0860693707700E-07
7.8025564785000E-06
2.2833863601700E-04
3.2437733422400E-03
2.4810520887500E-02
1.0901720602000E-01
2.8667550536300E-01
4.6224366960100E-01
4.6224366960100E-01
2.8667550536300E-01
1.0901720602000E-01
2.4810520887500E-02
3.2437733422400E-03
2.2833863601700E-04
7.8025564785000E-06
1.0860693707700E-07
4.3993409922600E-10
2.2293936455400E-13
];

eps=permute(eps,[2,3,1]);
ww=permute(ww,[2,3,1]);

p_grid=linspace(0,.05,25);
k_grid=linspace(-.1,.1,25);
[p_grid,k_grid]=ndgrid(p_grid,k_grid);

C1 =  cbar + G1(c,CONS) + k_grid*G1(c,k) + impact(c,1)*p_grid;
W1 =  wbar + G1(wc,CONS) + k_grid*G1(wc,k) + impact(wc,1)*p_grid;
K1=exp(k_grid+kbar);

K2 = (1-dep)*K1 + (K1.^gamma) - exp(C1);

p_grid2=bsxfun(@plus,phi_p*p_grid + (1-phi_p)*p_bar,eps*impact(p,1));

C2 = cbar + G1(c,CONS) + bsxfun(@plus,log(K2/Kbar)*G1(c,k),...
                        impact(c,1)*p_grid2);

W2 = wbar + G1(wc,CONS) + bsxfun(@plus,log(K2/Kbar)*G1(wc,k),...
                        impact(wc,1)*p_grid2);
                
Rw2 = bsxfun(@times,(exp(W2)+1).*exp(C2),1./(exp(W1+C1)));

p_orig=p;

p=p_grid;

Rk2 = gamma*K2.^(gamma-1);
Rk2=repmat(Rk2,[1,1,1,2]);
Rk2(:,:,:,1)=bsxfun(@plus,Rk2(:,:,:,1),(1-dep)*exp(DIS.*p./(1-p)));
Rk2(:,:,:,2)=bsxfun(@plus,Rk2(:,:,:,2),(1-dep)*exp(-DIS));

Rw2=repmat(Rw2,[1,1,1,2]);
Rw2(:,:,:,1)=bsxfun(@times,Rw2(:,:,:,1),exp(DIS.*p./(1-p)));
Rw2(:,:,:,2)=bsxfun(@times,Rw2(:,:,:,2),exp(-DIS));

CC=exp(bsxfun(@plus,C2,-C1));
CC=repmat(CC,[1,1,1,2]);
CC(:,:,:,1)=bsxfun(@times,CC(:,:,:,1),exp(DIS.*p./(1-p)));
CC(:,:,:,2)=bsxfun(@times,CC(:,:,:,2),exp(-DIS));

EE_rw = exp(-beta*((1-alpha)/(1-rho)) - rho*((1-alpha)/(1-rho))*log(CC) + ((1-alpha)/(1-rho))*log(Rw2));
EE_rw=bsxfun(@times,1-p,EE_rw(:,:,:,1)) + bsxfun(@times,p,EE_rw(:,:,:,2));
EE_rw=sum(bsxfun(@times,EE_rw,ww),3);
% meshc(log10(abs(EE_rw'-1)))

EE_rk = exp(-beta*((1-alpha)/(1-rho)) - rho*((1-alpha)/(1-rho))*log(CC) + ((rho-alpha)/(1-rho))*log(Rw2));
EE_rk=bsxfun(@times,EE_rk,Rk2);
EE_rk=bsxfun(@times,1-p,EE_rk(:,:,:,1)) + bsxfun(@times,p,EE_rk(:,:,:,2));
EE_rk=sum(bsxfun(@times,EE_rk,ww),3);

%%%%%%%%%%%%%%%%%%%%%%%%
Xs=zeros(9,100000);
Xs(:,1)=SS;
for i=2:100000
    Xs(:,i)=G1*Xs(:,i-1) + impact*[randn;0];
end


% %%%%%%%%%%%%%%%%%%%%%%%%
Xs=zeros(9,100);
Xs(:,1)=SS;
Xs(:,2)=SS+impact(:,1);
for i=3:100
    Xs(:,i)=G1*Xs(:,i-1);
end

iinv=log(exp(gamma*(Xs(k,:)+kbar)) - exp(Xs(c,:)+cbar));
(Xs(rw,:)-Xs(rf,:))'-(Xs(rw,1)-Xs(rf,1))


errors=interpn(p_grid,k_grid,EE_rk,Xs(p_orig,:),Xs(k,:));
errors=errors(~isnan(errors));






