
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% now we do it with epstein--zin preferences with potentially 
%% time-varying risk aversion

clear all

dep=.02;
gamma=.33;
rho=2/3;
beta=-log(.99)/4;
phi=.9;

rbar=beta;
Kbar=((exp(beta)-1+dep)/gamma)^(1/(gamma-1));

kbar=log(Kbar);
Ibar = dep*Kbar;
cbar = log((Kbar^gamma) - Ibar);

wbar=-log(exp(rbar)-1);
phi_z=.95;
alpha_bar=15;
zbar = (1-alpha_bar)/(1-rho);

%% these will be coordinates in the matrices GAM0 and GAM1 for gensys
c=1;
k=2;
z=3;
rw=4;
wc=5;
erw=6; %% erw, ec, and ea are expectational variables
ec=7;
a=8;
rc=9;
erc=10;
wcc=11;
CONS=12;

GAM0=zeros(10,10); GAM1=GAM0;

%% capital euler equation
GAM0(1,erw) = -1;
GAM0(1,k) = gamma*(gamma-1)*exp((gamma-1)*kbar)/...
    (gamma*exp((gamma-1)*kbar)+1-dep);

%% budget constraint
GAM0(2,c) = -exp(cbar);
GAM0(2,k) = -exp(kbar);
GAM0(2,a) = -exp(cbar)-exp(kbar) - (1-gamma)*exp(gamma*kbar);
GAM1(2,k) = -(1-dep)*exp(kbar)-gamma*exp(gamma*kbar);

%% euler equation for wealth
GAM0(3,erw) = 1;
GAM0(3,ec) = -rho;
GAM0(3,c) = rho;

%% wealth return
GAM0(4,rw) = exp(rbar);
GAM0(4,c) = -(exp(wbar)+1)/exp(wbar);
GAM1(4,c) = -(exp(wbar)+1)/exp(wbar);
GAM0(4,a) = -(exp(wbar)+1)/exp(wbar);
GAM0(4,wc) = -1;
GAM1(4,wc) = -(exp(wbar)+1)/exp(wbar);

%% risk aversion (transformed: z=(1-alpha)/(1-rho) )
GAM0(5,z) = 1;
GAM1(5,z) = phi_z;
GAM0(5,CONS)=-(1-phi_z)*zbar;

%% technology
GAM0(6,a)=1;

%% consumption claim
LL=2;
GAM0(7,rc) = exp(rbar);
GAM0(7,c) = -LL*(exp(wbar)+1)/exp(wbar);
GAM0(7,a) = -LL*(exp(wbar)+1)/exp(wbar);
GAM0(7,wcc) = -1;

GAM1(7,c) = -LL*(exp(wbar)+1)/exp(wbar);
GAM1(7,wcc) = -(exp(wbar)+1)/exp(wbar);

GAM0(8,erc) = 1;
GAM0(8,erw) = -1;

%% expectational equations 
GAM0(9,c)=1; GAM1(9,ec)=1;
GAM0(10,rw)=1; GAM1(10,erw)=1;
GAM0(11,rc)=1; GAM1(11,erc)=1;

%% constant
GAM0(12,CONS)=1;
GAM1(12,CONS)=1;

%% innovations; scaled so that the errors have unit variance
PSI=zeros(12,2);

%% innovation to technology
PSI(6,1)=1; 
%% innovation to z
PSI(5,2)=1; 

sd_Z=(alpha_bar/(1-rho))/2;
i_z=sd_Z*sqrt(1-phi_z);

SD=diag([.01,i_z]);
SIG=SD'*SD;

%% expectational errors
PPI=zeros(12,3);
PPI(9:11,:)=eye(3);

%% solve the first time (first-order perturbation)
C0=zeros(12,1);
[G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(GAM0,GAM1,C0,PSI,PPI) ;

%% get an impulse response function
X=zeros(12,1); X(z)=1;
IRF=zeros(12,100);
for i=1:100
    IRF(:,i)=(G1^(i-1))*X;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% now we find what the coefficient on z should be

Gk=zeros(1,12); Gk(k)=1;
Gc=zeros(1,12); Gc(c)=1; Gc(a)=1;
Gw=zeros(1,12); Gw(rw)=1;
Ga=zeros(1,12); Ga(a)=1;
Gz=zeros(1,12); Gz(z)=1;


for i=1:10

  
    fxp = Ga*gamma*(1-gamma)*exp((gamma-1)*kbar)/(gamma*exp((gamma-1)*kbar)+1-dep);

    ka = (-rho*Gc+Gw)*impact*SIG*impact'*(-Gw+fxp)';
    kc = (1/2)*(-Gw + fxp)*impact*SIG*impact'*(-Gw + fxp)' ;
    
    rca = (-rho*Gc+Gw)*impact*SIG*impact'*(-Gw)';
%     0 = -erw + rf + rfa*z
%     erw-rf = rfa*z
%      -- z up=RRA down=excess return down= rfa*z down, so rfa<0

%     0 = -erw + rk + ka*z
%     0 = rho*(ec-c) - wca*z + rk + ka*z
%     rho*Edc = (wca-ka)*z + rk
%     
%     (wca-ka)>0. So when z rises, Edc also rises, and consumption falls.
      
    wca = (1/2)*(-rho*Gc+Gw)*impact*SIG*impact'*(-rho*Gc+Gw)';

    fxp = zeros(1,12); fxp(rc)=1;
    rca = (-rho*Gc+Gw)*impact*SIG*impact'*(-Gw+fxp)';
    rcc = (1/2)*(-Gw + fxp)*impact*SIG*impact'*(-Gw + fxp)' ;
    
%% capital euler equation
    GAM0(1,erw) = -1;
    GAM0(1,k) = gamma*(gamma-1)*exp((gamma-1)*kbar)/...
        (gamma*exp((gamma-1)*kbar)+1-dep);
    GAM0(1,z) = ka;
    GAM0(1,CONS) = kc;
    
    %% euler equation for wealth
    GAM0(3,erw) = 1;
    GAM0(3,ec) = -rho;
    GAM0(3,c) = rho;
    GAM0(3,z) = wca;
    
    %% euler equation for rf
    GAM0(8,erc) = 1;
    GAM0(8,erw) = -1;
    GAM0(8,z) = rca;
    GAM0(8,CONS) = rcc;

    disp([ka kc wca])

    [G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(GAM0,GAM1,C0,PSI,PPI) ;

end

%% get an impulse response function

IRF=zeros(12,100);
for i=1:100
    IRF(:,i)=(G1^(i-1))*impact(:,2)*SD(2,2);
end

%% steady state
SS = (G1^1000)*[ones(11,1); 1];

HJ = (-rho*SS(z)*Gc + (SS(z)-1)*Gw)*impact*SD;

eps=sqrt(2)*[-2.350604974
-1.335849074
-0.436077412
0.436077412
1.335849074
2.350604974
];

ww=[2.555784E-03
8.861575E-02
4.088285E-01
4.088285E-01
8.861575E-02
2.555784E-03
];

epsz=permute(eps,[3,4,1,2])*SD(2,2);
epsa=permute(eps,[3,4,2,1])*SD(1,1);

wwz=permute(ww,[3,4,1,2]);
wwa=permute(ww,[3,4,2,1]);

z_grid=linspace(-1,2*SS(z),25);
k_grid=linspace(-.1,.1,25);
[z_grid,k_grid]=ndgrid(z_grid,k_grid);

C1 =  cbar + G1(c,end) + k_grid*G1(c,k) + impact(c,2)*z_grid;
W1 =  wbar + G1(wc,end) + k_grid*G1(wc,k) + impact(wc,2)*z_grid;
WC1 =  wbar + G1(wcc,end) + k_grid*G1(wcc,k) + impact(wcc,2)*z_grid;
K1=exp(k_grid+kbar);
a_grid=0*z_grid;

K2 = (1-dep)*K1.*exp(-a_grid) + exp(-gamma*a_grid).*(K1.^gamma) - exp(C1);

z_grid2=bsxfun(@plus,phi_z*z_grid + (1-phi_z)*zbar,epsz*impact(z,2));
a_grid2=epsa*impact(a,1);
C2 = cbar + G1(c,end) + bsxfun(@plus,log(K2/Kbar)*G1(c,k),...
                        bsxfun(@plus,impact(c,2)*z_grid2,...
                                    impact(c,1)*a_grid2));

W2 = wbar + G1(wc,end) + bsxfun(@plus,log(K2/Kbar)*G1(wc,k),...
                        bsxfun(@plus,impact(wc,2)*z_grid2,...
                                    impact(wc,1)*a_grid2));
WC2 = wbar + G1(wcc,end) + bsxfun(@plus,log(K2/Kbar)*G1(wcc,k),...
                        bsxfun(@plus,impact(wcc,2)*z_grid2,...
                                    impact(wcc,1)*a_grid2));
                                
Rw2 = bsxfun(@times,bsxfun(@times,(exp(W2)+1).*exp(C2),1./(exp(W1+C1))),exp(epsa));
Rc2 = bsxfun(@times,bsxfun(@times,(exp(WC2)+1).*exp(2*C2),1./(exp(WC1+2*C1))),exp(2*epsa));

Rk2 = bsxfun(@times,exp(epsa*(1-gamma)),gamma*K2.^(gamma-1))+1-dep;

EE_rw = exp(bsxfun(@times,z_grid,(-beta - rho*bsxfun(@plus,bsxfun(@plus,C2,-C1),epsa) + log(Rw2))));
EE_rw=sum(sum(bsxfun(@times,bsxfun(@times,EE_rw,wwz),wwa),3),4);
% meshc(log10(abs(EE_rw'-1)))

EE_rk = exp(bsxfun(@plus,...
                    bsxfun(@times,z_grid,(-beta - rho*bsxfun(@plus,bsxfun(@plus,C2,-C1),epsa) + log(Rw2)))-log(Rw2)...
                   , log(Rk2)));
EE_rk=sum(sum(bsxfun(@times,bsxfun(@times,EE_rk,wwz),wwa),3),4);
% meshc(log10(abs(EE_rk'-1)))

EE_rc = exp(bsxfun(@plus,...
                    bsxfun(@times,z_grid,(-beta - rho*bsxfun(@plus,bsxfun(@plus,C2,-C1),epsa) + log(Rw2)))-log(Rw2)...
                   , log(Rc2)));
EE_rc=sum(sum(bsxfun(@times,bsxfun(@times,EE_rc,wwz),wwa),3),4);
% meshc(log10(abs(EE_rc'-1)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% now we simulate the model and interpolate for the error at each point

Xs=zeros(12,100000);
Xs(:,1)=SS;
for i=2:100000
    Xs(:,i)=G1*Xs(:,i-1) + impact*SD*randn(2,1);
end

Err=log10(abs(EE_rk-1));
errors=interpn(z_grid,k_grid,Err,Xs(z,:),Xs(k,:));

Xs=Xs(:,~isnan(errors));
errors=errors(~isnan(errors));

